﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.IO;
using System.Drawing;
using MindFusion.Drawing;
using MindFusion.Diagramming;
using MindFusion.Diagramming.Layout;
using MindFusion.Diagramming.Mvc;
using Pen = MindFusion.Drawing.Pen;
using SolidBrush = MindFusion.Drawing.SolidBrush;
using Brush = MindFusion.Drawing.Brush;

namespace DirTree.Controllers
{
    public class HomeController : Controller
    {
        public ActionResult Index()
        {
            DiagramView view = new DiagramView("diagramView1");
            Diagram diagram = view.Diagram;

            // all arrows should start with vertical segment
            diagram.LinkCascadeOrientation = MindFusion.Diagramming.Orientation.Vertical;
            diagram.LinkHeadShape = ArrowHeads.BowArrow;
            diagram.LinkHeadShapeSize = 3;
            diagram.DefaultShape = Shapes.Rectangle;

            view.Behavior = Behavior.Pan;

            DiagramStyle style = new DiagramStyle();
            style.ShadowBrush = null;
            style.Brush = new MindFusion.Drawing.SolidBrush(ColorTranslator.FromHtml("#e0e9e9"));
            style.NodeEffects.Add(new AeroEffect());
            diagram.Style = style;

            // read some directories
            BuildTree(diagram);

            ViewBag.DiagramView = view;
            return View();
        }

        int height = 0;

        private void BuildTree(Diagram diagram)
        {
            // create a box that will be a root in our hierarchy
            ShapeNode root = new ShapeNode(diagram);
            root.Bounds = new RectangleF(6, 6, 26, 12);
            root.Text = "root";
            diagram.Nodes.Add(root);

            height = 20;

            // read directories of all drives
            string[] drives = Environment.GetLogicalDrives();

            int skip = 0;
            if (drives.Length >= 2)
            {
                if (drives[0][0] == 'A') skip++;
                if (drives[1][0] == 'B') skip++;
            }

            DirectoryInfo[] di = new DirectoryInfo[drives.Length - skip];
            for (int i = 0; i < di.GetLength(0); ++i)
            {
                di[i] = new DirectoryInfo(drives[i + skip]);
            }

            BuildSubDirs(diagram, root, di, 1);
            PerformLayout(diagram);

            // after the whole tree is built adjust the document extents
            // to be as big as needed
            diagram.Bounds = new RectangleF(0, 0, diagram.Bounds.Width, diagram.Nodes[diagram.Nodes.Count - 1].Bounds.Bottom);
        }

        private void PerformLayout(Diagram diagram)
        {
            TreeLayout tl = new TreeLayout();

            tl.Root = null; // automatically select root
            tl.Type = TreeLayoutType.Cascading;
            tl.Direction = TreeLayoutDirections.LeftToRight;
            tl.LinkStyle = TreeLayoutLinkType.Cascading2;
            tl.LevelDistance = -6;
            tl.NodeDistance = 5;
            tl.KeepRootPosition = true;
            tl.ReversedLinks = false;

            tl.Arrange(diagram);
        }

        private void BuildSubDirs(Diagram diagram, ShapeNode root, DirectoryInfo[] dirs, int level)
        {
            foreach (DirectoryInfo dir in dirs)
            {
                // create a node for the subfolder
                ShapeNode folder = new ShapeNode(diagram);
                folder.Bounds = new RectangleF(6 + level * 20, height, 26, 12);
                folder.Text = dir.Name.Replace(@":\", "");
                diagram.Nodes.Add(folder);

                // link the subfolder with its parent
                DiagramLink link = new DiagramLink(diagram, root, folder);
                link.SegmentCount = 2;
                link.Shape = LinkShape.Cascading;
                diagram.Links.Add(link);

                // vertical offset for next folders
                height += 14;
                if (level > 1) continue; // avoid deep recursion

                try
                {
                    // build subtrees recursively
                    DirectoryInfo[] subDirs = dir.GetDirectories();
                    BuildSubDirs(diagram, folder, subDirs, level + 1);
                }
                catch (Exception)
                {
                }
            }
        }
    }
}